
#define	IDR_VIM 150

#define IDR_VIM_ERROR	151
#define IDR_VIM_ALERT	152
#define IDR_VIM_INFO	153
#define IDR_VIM_QUESTION 154

#define IDR_ICOBUDDYBASE			200

#define IDR_ICOBUDDY_DEF1			(IDR_ICOBUDDYBASE + 3)
#define IDR_ICOBUDDY_DEF2			(IDR_ICOBUDDYBASE + 0)
#define IDR_ICOBUDDY_DEF3			(IDR_ICOBUDDYBASE + 1)
#define IDR_ICOBUDDY_DEF4			(IDR_ICOBUDDYBASE + 2)

#define IDR_ICOBUDDY_GRIN			(IDR_ICOBUDDYBASE + 4)
#define IDR_ICOBUDDY_ALARM			(IDR_ICOBUDDYBASE + 5)
